@@ -37,10 +37,8 @@ aalen<-function (formula = formula(data),
   if(is.null(clusters)) clusters <- des$clusters ##########
   pxz <- px + pz; 
 
-  survs<-read.surv(m,id,npar,clusters,start.time,max.time)
-  times<-survs$times;
-  id<-survs$id.cal;
-  id.call<-id; 
+  survs<-read.surv(m,id,npar,clusters,start.time,max.time,silent=silent)
+  times<-survs$times; id<-survs$id.cal; id.call<-id; 
   clusters<-cluster.call<-survs$clusters; 
   stop.call <- time2<-survs$stop
   start.call <- survs$start
@@ -65,10 +63,10 @@ aalen<-function (formula = formula(data),
    X<-as.matrix(X[ot,])
    if (npar==FALSE) Z<-as.matrix(Z[ot,])
    survs$stop<-time2;
-   ###print(cbind(X,time2,status,id))
    clusters<-clusters[ot]
    id<-id[ot];
    entry=rep(-1,nobs); 
+   weights <- weights[ot]
    if (sum(offsets)!=0) offsets <- offsets[ot]
   } else {
         eventtms <- c(survs$start,time2)
@@ -145,6 +143,8 @@ ldata<-list(start=survs$start,stop=survs$stop,
   attr(ud, "start.time") <- start.time
   attr(ud, "stop") <- stop.call
   attr(ud, "start") <- start.call
+  attr(ud, "status") <- survs$status
+  attr(ud, "residuals") <- residuals
   class(ud) <- "aalen"
   ud$call<-call
   return(ud)
